%==========================================================================
%    Writing .h5 file according to JONAS Exchange Data Format Proposal
%==========================================================================
%Function description:
%This function permit to write a ".h5" file according to the fields
%defined at the JONAS Exchange Data Format proposal.
%
%See the following list of fields:
%format_version 
%author 
%institution
%country_code
%contact
%start_date
%end_date
%date_of_creation
%purpose
%data_uuid
%data_type
%comments
%expsetup
%recorder_manufacturer
%recorder_serial_number
%recorder_model
%builtin_hydrophone
%hydrophone_manufacturer
%hydrophone_sensitivity
%hydrophone_serial_number
%hydrophone_model
%calibration_frequency_count
%calibration_datetime
%calibration_factor
%calibration_procedure
%reference_frequencies
%ais_database
%source_levels
%bathymetry_database
%temperature_database
%salinity_database
%sound_speed_profile_model
%propagation_model
%numeric_data_calibration
%hydrophone_count
%total_number_of_grid_points
%longitude
%latitude
%depth
%frequency_count
%frequency_band_definition
%frequency
%time_duty_on
%time_duty_off
%time_count
%time
%averaging_time
%spl_values
%percentile_list
%percentile_count
%percentile
%--------------------------------------------------------------------------
%The complete field description can be seen at the JONAS Deliverable D4.3 
%available at: xxxxxxxxxx
%
%Note:
%To call this function the user should write the name of the field followed  
%by the variable content as the examples below:
%--------------------------------------------------------------------------
%
%Example 1 - Generic description:
%
%write_edf(FIELDNAME1,'USERINPUT1',FIELDNAME1,'USERINPUT1',...)
%
%FIELDNAME corresponds to the fields detailed on the list of fields and 
%completly described at the JONAS Exchange Data Format proposal. 
%USERINPUT corresponds to the content introduced by the user in each field. 
%
%Example 2:
%Consider these three fields: file name;institution and spl values.
%
%Step 1: defining the variables
%h5filename='example.h5';
%institution='UALG'
%spl_values=rand(10); %generates a matrix of 10x10 random values 
%between 0 and 1
%
%Step 2: calling the function
%write_edf('new_file_name',h5filename,'institution',institution,'spl_values',spl_values);
%
%
%Example 3:
%Considering all fiedls in the case of numerical input:
%
%Step 1: defining the variables
%h5filename='example3.h5'
%format_version='EDF_version1.1'; 
%author='RD'; 
%institution='UALG';
%country_code='PT';
%contact='info@siplab.fct.ualg.pt';
%start_date=01012020;
%end_date=01012020;
%date_of_creation=01012020;
%purpose='test standalone routines';
%data_uuid='N/A';
%data_type='Numerical';
%comments='testing purpose edf standalone routines';
%expsetup=[];
%recorder_manufacturer=[];
%recorder_serial_number=[];
%recorder_model=[];
%builtin_hydrophone=[];
%hydrophone_manufacturer=[];
%hydrophone_sensitivity=[];
%hydrophone_serial_number=[];
%hydrophone_model=[];
%calibration_frequency_count=[];
%calibration_datetime=[];
%calibration_factor=[];
%calibration_procedure=[];
%reference_frequencies=[];
%ais_database='Exact Hearth';
%source_levels=[50 60 80; 170 120 180];
%bathymetry_database='GEBCO';
%temperature_database='COPERNICUS';
%salinity_database='COPERNICUS';
%sound_speed_profile_model='Mackenzie nine-term equation';
%propagation_model='Kraken';
%numeric_data_calibration=[];
%hydrophone_count=[];
%total_number_of_grid_points=[1];
%longitude=[40.446,41.115];
%latitude=[79.982,81.281];
%depth=[10];
%frequency_count=2;
%frequency_band_definition='1/3octave-band levels';
%frequency=[63 126];
%time_duty_on=[];
%time_duty_off=[];
%time=[1562151105,1562153100];
%time_count=length(time);
%averaging_time=[1];
%spl_values=rand(10);
%percentile_list=[5 10 15 25 50 75 90 95];
%percentile_count=length(percentile_list);
%percentile=rand(10,7);
%
%Step 2: calling the function
%write_edf('new_file_name',h5filename,'format_version',format_version,... 
%'author',author,'institution',institution,'country_code',country_code,...
%'contact',contact,'start_date',start_date,'end_date',end_date,...
%'date_of_creation',date_of_creation,'purpose',purpose,'data_uuid',data_uuid,...
%'data_type',data_type,'comments',comments,'ais_database',ais_database,...
%'source_levels',source_levels,'bathymetry_database',bathymetry_database,...
%'temperature_database',temperature_database,'salinity_database',salinity_database,...
%'sound_speed_profile_model',sound_speed_profile_model,...
%'propagation_model',propagation_model,'numeric_data_calibration',numeric_data_calibration,...
%'total_number_of_grid_points',total_number_of_grid_points,...
%'longitude',longitude,'latitude',latitude,'depth',depth,...
%'frequency_count',frequency_count,'frequency_band_definition',frequency_band_definition,...
%'frequency',frequency,'time_duty_on',time_duty_on,'time_duty_off',time_duty_off,...
%'time_count',time_count,'time',time,'averaging_time',averaging_time,...
%'spl_values',spl_values,'percentile_list',percentile_list,...
%'percentile_count',percentile_count,'percentile',percentile)
%
%
%
%See also read_edf


%--------------------------------------------------------------------------

function write_edf(varargin)

new_file_name='teste.h5';
format_version=[];
author=[];
institution=[];
country_code=[];
contact=[];
start_date=[];
end_date=[];
date_of_creation=[];
purpose=[];
data_uuid=[];
data_type=[];
comments=[];
%--------------------
expsetup=[];
%--------------------
recorder_manufacturer=[];
recorder_serial_number=[];
recorder_model=[];
builtin_hydrophone=[];
%--------------------
hydrophone_manufacturer=[];
hydrophone_sensitivity=[];
hydrophone_serial_number=[];
hydrophone_model=[];
%--------------------
calibration_frequency_count=0;
calibration_datetime=date;
calibration_factor=0;
calibration_procedure={'Not defined'};
reference_frequencies=0;
%--------------------
ais_database=[];
source_levels=0;
bathymetry_database=[];
temperature_database=[];
salinity_database=[];
sound_speed_profile_model=[];
propagation_model=[];
numeric_data_calibration=[];
%--------------------
hydrophone_count=1;
total_number_of_grid_points=1;
longitude=0;
latitude=0;
depth=0;
%--------------------
frequency_count=0;
frequency_band_definition=0;
frequency=0;
%--------------------
time_duty_on=[];
time_duty_off=[];
time_count=[];
time=[0];
%--------------------
averaging_time=[];
spl_values=0;
%--------------------
percentile_list=0;
percentile_count=length(percentile_list);
percentile=0;
%--------------------

for opt=1:2:nargin
    switch varargin{opt}
        case 'new_file_name'
            new_file_name=varargin{opt+1};
        case 'format_version'
            format_version=varargin{opt+1};
        case 'author'
            author=varargin{opt+1};
        case 'institution'
            institution=varargin{opt+1};
        case 'country_code'
            country_code=varargin{opt+1};
        case 'contact'
            contact=varargin{opt+1};
        case 'start_date'
            start_date=varargin{opt+1};
        case 'end_date'
            end_date=varargin{opt+1};
        case 'date_of_creation'
            date_of_creation=varargin{opt+1};
        case 'purpose'
            purpose=varargin{opt+1};
        case 'data_uuid'
            data_uuid=varargin{opt+1};
        case 'data_type'
            data_type=varargin{opt+1};
        case 'comments'
            comments=varargin{opt+1};
        case 'expsetup'
            expsetup=varargin{opt+1};
        case 'recorder_manufacturer'
            recorder_manufacturer=varargin{opt+1};
        case 'recorder_serial_number'
            recorder_serial_number=varargin{opt+1};
        case 'recorder_model'
            recorder_model=varargin{opt+1};
        case 'buildin_hydrophone'
            buildin_hydrophone=varargin{opt+1};
        case 'hydrophone_manufacturer'
            hydrophone_manufacturer=varargin{opt+1};
        case 'hydrophone_sensitivity'
            hydrophone_sensitivity=varargin{opt+1};
        case 'hydrophone_serial_number'
            hydrophone_serial_number=varargin{opt+1};
        case 'hydrophone_model'
            hydrophone_model=varargin{opt+1};
        case 'calibration_frequency_count'
            calibration_frequency_count=varargin{opt+1};
        case 'calibration_datetime'
            calibration_datetime=varargin{opt+1};
        case 'calibration_factor'
            calibration_factor=varargin{opt+1};
        case 'calibration_procedure'
            calibration_procedure=varargin{opt+1};
        case 'reference_frequencies'
            expsetup=varargin{opt+1};
        case 'ais_database'
            ais_database=varargin{opt+1};
        case 'source_levels'
            source_levels=varargin{opt+1};
        case 'bathymetry_database'
            bathymetry_database=varargin{opt+1};
        case 'temperature_database'
            temperature_database=varargin{opt+1};
        case 'salinity_database'
            salinity_database=varargin{opt+1};
        case 'sound_speed_profile_model'
            sound_speed_profile_model=varargin{opt+1};
        case 'propagation_model'
            propagation_model=varargin{opt+1};
        case 'numeric_data_calibration'
            numeric_data_calibration=varargin{opt+1};
        case 'hydrophone_count'
            hydrophone_count=varargin{opt+1};
        case 'total_number_of_grid_points'
            total_number_of_grid_points=varargin{opt+1};
        case 'longitude'
            longitude=varargin{opt+1};
        case 'latitude'
            latitude=varargin{opt+1};
        case 'depth'
            depth=varargin{opt+1};
        case 'frequency_count'
            frequency_count=varargin{opt+1};
        case 'frequency_band_definition'
            frequency_band_definition=varargin{opt+1};
        case 'frequency'
            frequency=varargin{opt+1};
        case 'time_duty_on'
            time_duty_on=varargin{opt+1};
        case 'time_duty_off'
            time_duty_off=varargin{opt+1};
        case 'time_count'
            time_count=varargin{opt+1};
        case 'time'
            time=varargin{opt+1};
        case 'averaging_time'
            averaging_time=varargin{opt+1};
        case 'spl_values'
            spl_values=varargin{opt+1};
        case 'percentile_count'
            percentile_count=varargin{opt+1};
        case 'percentile_list'
            percentile_list=varargin{opt+1};
        case 'percentile'
            percentile=varargin{opt+1};
    end
end

            
fid = H5F.create(new_file_name);
plist = 'H5P_DEFAULT';

gid1 = H5G.create(fid,'analysis_metadata',plist,plist,plist);
gid2 = H5G.create(gid1,'experimental',plist,plist,plist);
gid3 = H5G.create(gid2,'recorder',plist,plist,plist);
gid4 = H5G.create(gid2,'hydrophone',plist,plist,plist);
gid5 = H5G.create(gid2,'calibration',plist,plist,plist);
gid6 = H5G.create(gid1,'numerical_model',plist,plist,plist);
gid7 = H5G.create(fid,'ambient_noise_dataset',plist,plist,plist);
gid8 = H5G.create(gid7,'time',plist,plist,plist);
gid9 = H5G.create(gid7,'sound_pressure_levels',plist,plist,plist);

H5G.close(gid9);
H5G.close(gid8);
H5G.close(gid7);
H5G.close(gid6);
H5G.close(gid5);
H5G.close(gid4);
H5G.close(gid3);
H5G.close(gid2);
H5G.close(gid1);
H5F.close(fid);
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%                           GLOBAL ATTRIBUTES
%--------------------------------------------------------------------------
h5writeatt(new_file_name,'/', 'format_version',format_version);
h5writeatt(new_file_name,'/','author',author);
h5writeatt(new_file_name,'/','institution',institution); 
h5writeatt(new_file_name,'/','country_code',country_code); 
h5writeatt(new_file_name,'/','contact',contact); 
h5writeatt(new_file_name,'/','start_date',start_date);
h5writeatt(new_file_name,'/','end_date',end_date);
h5writeatt(new_file_name,'/','date_of_creation', date_of_creation);
h5writeatt(new_file_name,'/','purpose',purpose);
h5writeatt(new_file_name,'/','data_uuid',data_uuid);
h5writeatt(new_file_name,'/','data_type',data_type);
h5writeatt(new_file_name,'/','comments',comments);
%--------------------------------------------------------------------------
%                    ANALYSIS METADATA / EXPERIMENTAL
%--------------------------------------------------------------------------
h5writeatt(new_file_name,'/analysis_metadata/experimental','exsetup',expsetup)
%--------------------------------------------------------------------------
%              ANALYSIS METADATA / EXPERIMENTAL / RECORDER
%--------------------------------------------------------------------------
h5writeatt(new_file_name,'/analysis_metadata/experimental/recorder','recorder_manufacturer',recorder_manufacturer);
h5writeatt(new_file_name,'/analysis_metadata/experimental/recorder','recorder_serial_number',recorder_serial_number);
h5writeatt(new_file_name,'/analysis_metadata/experimental/recorder','recorder_model',recorder_model);
h5writeatt(new_file_name,'/analysis_metadata/experimental/recorder','builtin_hydrophone',builtin_hydrophone);
%--------------------------------------------------------------------------
%              ANALYSIS METADATA / EXPERIMENTAL / HYDROPHONE
%--------------------------------------------------------------------------
h5writeatt(new_file_name,'/analysis_metadata/experimental/hydrophone','hydrophone_manufacturer',hydrophone_manufacturer);
h5writeatt(new_file_name,'/analysis_metadata/experimental/hydrophone','hydrophone_sensitivity',hydrophone_sensitivity);
h5writeatt(new_file_name,'/analysis_metadata/experimental/hydrophone','hydrophone_serial_number',hydrophone_serial_number);
h5writeatt(new_file_name,'/analysis_metadata/experimental/hydrophone','hydrophone_model',hydrophone_model);
%--------------------------------------------------------------------------
%              ANALYSIS METADATA / EXPERIMENTAL / CALIBRATION
%--------------------------------------------------------------------------
%CALIBRATION_FREQUENCY_COUNT
h5create(new_file_name,'/analysis_metadata/experimental/calibration/calibration_frequency_count',size(calibration_frequency_count)); 
h5write(new_file_name,'/analysis_metadata/experimental/calibration/calibration_frequency_count',calibration_frequency_count);
%CALIBRATION_DATETIME
hdf5write(new_file_name, '/analysis_metadata/experimental/calibration/calibration_datetime', calibration_datetime,'WriteMode', 'append');
%CALIBRATION_FACTOR
h5create(new_file_name,'/analysis_metadata/experimental/calibration/calibration_factor',size(calibration_factor)); 
h5write(new_file_name,'/analysis_metadata/experimental/calibration/calibration_factor',calibration_factor);
%CALIBRATION_PROCEDURE
hdf5write(new_file_name, '/analysis_metadata/experimental/calibration/calibration_procedure', calibration_procedure,'WriteMode', 'append');
%REFERENCE_FREQUENCIES
h5create(new_file_name,'/analysis_metadata/experimental/calibration/reference_frequencies',size(reference_frequencies)); 
h5write(new_file_name,'/analysis_metadata/experimental/calibration/reference_frequencies',reference_frequencies);
%--------------------------------------------------------------------------
%                   ANALYSIS METADATA / NUMERICAL MODEL
%--------------------------------------------------------------------------
h5writeatt(new_file_name,'/analysis_metadata/numerical_model','ais_database',ais_database);
%SOURCE LEVELS
h5create(new_file_name,'/analysis_metadata/numerical_model/source_levels',size(source_levels)); 
h5write(new_file_name,'/analysis_metadata/numerical_model/source_levels',source_levels);

h5writeatt(new_file_name,'/analysis_metadata/numerical_model','bathymetry_database',bathymetry_database);
h5writeatt(new_file_name,'/analysis_metadata/numerical_model','temperature_database',temperature_database);
h5writeatt(new_file_name,'/analysis_metadata/numerical_model','salinity_database',salinity_database);
h5writeatt(new_file_name,'/analysis_metadata/numerical_model','sound_speed_profile_model',sound_speed_profile_model);
h5writeatt(new_file_name,'/analysis_metadata/numerical_model','propagation_model',propagation_model);
h5writeatt(new_file_name,'/analysis_metadata/numerical_model','numeric_data_calibration',numeric_data_calibration);
%--------------------------------------------------------------------------
%                        AMBIENT NOISE DATASET - POSITION
%--------------------------------------------------------------------------
%HYDROPHONE COUNT
h5create(new_file_name,'/ambient_noise_dataset/position/hydrophone_count',size(hydrophone_count)); 
h5write(new_file_name,'/ambient_noise_dataset/position/hydrophone_count',hydrophone_count);
%TOTAL NUMBER OF GRID POINTS
h5create(new_file_name,'/ambient_noise_dataset/position/total_number_of_grid_points',size(total_number_of_grid_points)); 
h5write(new_file_name,'/ambient_noise_dataset/position/total_number_of_grid_points',total_number_of_grid_points);
%LONGITUDE
h5create(new_file_name,'/ambient_noise_dataset/position/longitude',size(longitude)); 
h5write(new_file_name,'/ambient_noise_dataset/position/longitude',longitude);
%LATITUDE
h5create(new_file_name,'/ambient_noise_dataset/position/latitude',size(latitude)); 
h5write(new_file_name,'/ambient_noise_dataset/position/latitude',latitude);
%DEPTH
h5create(new_file_name,'/ambient_noise_dataset/position/depth',size(depth)); 
h5write(new_file_name,'/ambient_noise_dataset/position/depth',depth);
%--------------------------------------------------------------------------
%                        AMBIENT NOISE DATASET - FREQUENCY
%--------------------------------------------------------------------------
%FREQUENCY COUNT
h5create(new_file_name,'/ambient_noise_dataset/frequency/frequency_count',size(frequency_count)); 
h5write(new_file_name,'/ambient_noise_dataset/frequency/frequency_count',frequency_count);
%FREQUENCY BAND DEFINITION
hdf5write(new_file_name, '/ambient_noise_dataset/frequency/frequency_band_definition', frequency_band_definition,'WriteMode', 'append');
%FREQUENCY
h5create(new_file_name,'/ambient_noise_dataset/frequency/frequency',size(frequency)); %Search for PAMGUIDE VARIABLE NAME
h5write(new_file_name,'/ambient_noise_dataset/frequency/frequency',frequency); %Search for PAMGUIDE VARIABLE NAME
%--------------------------------------------------------------------------
%                        AMBIENT NOISE DATASET - TIME
%--------------------------------------------------------------------------
%TIME DUTY ON
h5writeatt(new_file_name,'/ambient_noise_dataset/time','time_duty_on',time_duty_on);
%TIME DUTY OFF
h5writeatt(new_file_name,'/ambient_noise_dataset/time','time_duty_off',time_duty_off);
%TIME COUNT
h5create(new_file_name,'/ambient_noise_dataset/time/time_count',[1 1]); %Search for PAMGUIDE VARIABLE NAME
h5write(new_file_name,'/ambient_noise_dataset/time/time_count',length(time)); %Search for PAMGUIDE VARIABLE NAME
%TIME
h5create(new_file_name,'/ambient_noise_dataset/time/time',size(time)); %Search for PAMGUIDE VARIABLE NAME
h5write(new_file_name,'/ambient_noise_dataset/time/time',time); %Search for PAMGUIDE VARIABLE NAME
%--------------------------------------------------------------------------
%                 AMBIENT NOISE DATASET - SOUND PRESSURE LEVEL
%--------------------------------------------------------------------------
%AVERAGIN TIME
h5writeatt(new_file_name,'/ambient_noise_dataset/sound_pressure_levels','averaging_time',averaging_time);
%SPL VALUES
h5create(new_file_name,'/ambient_noise_dataset/sound_pressure_levels/spl_values',size(spl_values)); %Search for PAMGUIDE VARIABLE NAME
h5write(new_file_name,'/ambient_noise_dataset/sound_pressure_levels/spl_values',spl_values); %Search for PAMGUIDE VARIABLE NAME
%--------------------------------------------------------------------------
%             AMBIENT NOISE DATASET - SOUND PRESSURE LEVEL STATS
%--------------------------------------------------------------------------
%PERCENTILE COUNT
h5create(new_file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_count',size(percentile_count)); %Search for PAMGUIDE VARIABLE NAME
h5write(new_file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_count',percentile_count); %Search for PAMGUIDE VARIABLE NAME
%PERCENTILE LIST
h5create(new_file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_list',size(percentile_list));  %Search for PAMGUIDE VARIABLE NAME
h5write(new_file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_list',percentile_list); %Search for PAMGUIDE VARIABLE NAME
%PERCENTILE VALUES
percentile=prctile(spl_values,percentile_list);
h5create(new_file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_values',size(percentile)); 
h5write(new_file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_values', percentile); 

end
%==========================================================================
%--------------------------------------------------------------------------